package featureflag_test

import (
	"github.com/stretchr/testify/require"
	"testing"
	"time"

	gofeatureflag "github.com/thomaspoignant/go-feature-flag"
	"github.com/thomaspoignant/go-feature-flag/ffcontext"
	"github.com/thomaspoignant/go-feature-flag/retriever/fileretriever"
)

func Test_FeatureFlag_InvalidBehavior(t *testing.T) {
	ff, err := gofeatureflag.New(gofeatureflag.Config{
		Retriever: &fileretriever.Retriever{
			Path: "feature_flag.yaml", // YAML with the malformed query
		},
		PollingInterval: time.Second * 10,
	})
	require.NoError(t, err)
	defer ff.Close()

	params := ffcontext.NewEvaluationContext("user123")
	params.AddCustomAttribute("param1", "param1value1")
	params.AddCustomAttribute("param2", "param2value")

	value, err := ff.StringVariation("test-rule", params, "default-value")
	require.NoError(t, err)

	// The malformed query prevents the match, so the default value is returned
	require.Equal(t, "default-value", value)
}

func Test_FeatureFlag_ORConditionsFail(t *testing.T) {
	ff, err := gofeatureflag.New(gofeatureflag.Config{
		Retriever: &fileretriever.Retriever{
			Path: "feature_flag.yaml", // Valid YAML configuration
		},
		PollingInterval: time.Second * 10,
	})
	require.NoError(t, err)
	defer ff.Close()

	params := ffcontext.NewEvaluationContext("user123")
	params.AddCustomAttribute("param1", "param1value3") // This value is part of the OR conditions
	params.AddCustomAttribute("param2", "param2value")

	value, err := ff.StringVariation("test-rule", params, "default-value")
	require.NoError(t, err)

	// The OR conditions are not correctly evaluated, so the default value is returned
	require.Equal(t, "default-value", value)
}

func Test_FeatureFlag_Target2Works(t *testing.T) {
	ff, err := gofeatureflag.New(gofeatureflag.Config{
		Retriever: &fileretriever.Retriever{
			Path: "feature_flag.yaml", // Valid YAML configuration
		},
		PollingInterval: time.Second * 10,
	})
	require.NoError(t, err)
	defer ff.Close()

	params := ffcontext.NewEvaluationContext("user123")
	params.AddCustomAttribute("param1", "param1value4") // Matches target2 conditions
	params.AddCustomAttribute("param2", "param2value")

	value, err := ff.StringVariation("test-rule", params, "default-value")
	require.NoError(t, err)

	// The correct variation for target2 is returned
	require.Equal(t, "value2", value)
}
